{#- Device-Specific Port Declarations Template -#}
{%- if device_config.device_type.value == "network" %}
// Network controller ports
    output logic link_up,
    output logic [1:0] link_speed,
    input logic [7:0] phy_data,
    output logic [7:0] mac_data
{%- elif device_config.device_type.value == "storage" %}
// Storage controller ports
    output logic storage_ready,
    output logic [7:0] queue_depth,
    input logic [31:0] sector_addr,
    output logic [31:0] data_out
{%- elif device_config.device_type.value == "graphics" %}
// Graphics controller ports
    output logic display_active,
    output logic [7:0] gpu_utilization,
    input logic [23:0] pixel_data,
    output logic vsync, hsync
{%- else %}
// Generic device ports
    output logic device_ready,
    input logic [31:0] generic_input,
    output logic [31:0] generic_output
{%- endif %}