{#- Complete Error Handling Logic Template -#}
{%- include 'systemverilog/advanced/error_declarations.sv.j2' %}

{%- include 'systemverilog/advanced/error_detection.sv.j2' %}

{%- include 'systemverilog/advanced/error_state_machine.sv.j2' %}

{%- include 'systemverilog/advanced/error_logging.sv.j2' %}

{%- include 'systemverilog/advanced/error_counters.sv.j2' %}

{%- include 'systemverilog/advanced/error_injection.sv.j2' %}

{%- include 'systemverilog/advanced/error_outputs.sv.j2' %}