"""Version information for PCILeech Firmware Generator."""

__version__ = "0.5.8"
__version_info__ = (0, 5, 7)

# Release information
__title__ = "PCILeech Firmware Generator"
__description__ = "Generate spoofed PCIe DMA firmware from real donor hardware"
__author__ = "Ramsey McGrath"
__author_email__ = "ramsey@voltcyclone.info"
__license__ = "MIT"
__url__ = "https://github.com/ramseymcgrath/PCILeechFWGenerator"

# Build metadata
__build_date__ = "2025-06-05T01:44:07.732192"
__commit_hash__ = "39c13da"
